@@ -17,7 +17,12 @@
 
 #include <math.h>
 #include <string.h>
-#include <gnome.h>
+
+#include <gtk/gtkobject.h>
+#include <gtk/gtkwidget.h>
+#include <libgnomeui/gnome-canvas.h>
+#include <libgnomeui/gnome-canvas-util.h>
+
 #include "gnome-canvas-bpath.h"
 #include "gnome-canvas-bpath-private.h"
 #include "gp-path.h"
