@@ -118,7 +118,7 @@ gnome_font_selection_class_init (GnomeFontSelectionClass *klass)
 						GTK_SIGNAL_OFFSET (GnomeFontSelectionClass, font_set),
 						gtk_marshal_NONE__POINTER,
 						GTK_TYPE_NONE,
-						1, GTK_TYPE_POINTER);
+						1, GTK_TYPE_OBJECT);
 	gtk_object_class_add_signals (object_class, gfs_signals, LAST_SIGNAL);
 
 	object_class->destroy = gnome_font_selection_destroy;
