@@ -55,10 +55,12 @@
 #include <stdio.h>
 #include <math.h>
 #include <string.h>
+#include <libgnomeprint/gnome-printer.h>
 #include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print-pclr.h>
 #include <libgnomeprint/gnome-print-pclc.h>
 #include <libgnomeprint/gnome-print-encode.h>
+#include <libgnomeprint/gnome-print-encode-private.h>
 
 PCLJobData * jobdata;
 static GnomePrintRGBPClass *pclr_parent_class;
