@@ -311,6 +311,7 @@ gpp_glyphlist (GnomePrintContext *pc, GnomeGlyphList * glyphlist)
 	GnomePrintPreview *pp = GNOME_PRINT_PREVIEW (pc);
 	GnomeCanvasGroup *group;
 	GnomeCanvasItem *item;
+	const ArtPoint zeropoint = {0.0, 0.0};
 	const ArtPoint * cp;
 	ArtPoint p;
 	double transform [6], a[6], inverse[6];
@@ -322,7 +323,11 @@ gpp_glyphlist (GnomePrintContext *pc, GnomeGlyphList * glyphlist)
 	 * Postscript->Canvas translation
 	 */
 	
-	cp = gp_gc_get_currentpoint (pc->gc);
+	if (gp_gc_has_currentpoint (pc->gc)) {
+		cp = gp_gc_get_currentpoint (pc->gc);
+	} else {
+		cp = &zeropoint;
+	}
 
 	memcpy (transform, gp_gc_get_ctm (pc->gc), sizeof (transform));
 	art_affine_scale (a, 1.0, -1.0);
