@@ -6,6 +6,7 @@
  *   Raph Levien (raph@acm.org)
  *   Miguel de Icaza (miguel@kernel.org)
  *   Lauris Kaplinski <lauris@ariman.ee>
+ *   Chema Celorio (chema@celorio.com)
  */
 
 /*
@@ -26,16 +27,22 @@
 
 #include <string.h>
 #include <locale.h>
-
-#include <gtk/gtk.h>
 #include <stdio.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <sys/time.h>
+#include <errno.h>
 #include <libart_lgpl/art_affine.h>
 #include <libart_lgpl/art_vpath.h>
 #include <libart_lgpl/art_bpath.h>
-/* gnome.h is only needed for g_concat_string_and_file */
-#include <gnome.h>
+
+#include <libgnome/gnome-defs.h>
+#include <libgnome/gnome-util.h>
 
 #include <libgnomeprint/gp-unicode.h>
+#include <libgnomeprint/gnome-printer.h>
 #include <libgnomeprint/gnome-printer-private.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-private.h>
@@ -48,12 +55,22 @@
 #include <libgnomeprint/gnome-print-pclv.h>
 #include <libgnomeprint/gnome-print-pixbuf.h>
 #include <libgnomeprint/gnome-print-frgba.h>
+#include <libgnomeprint/gnome-print-fax.h>
 
-#include <libgnomeprint/gnome-font-private.h>
 #include <libgnomeprint/gnome-pgl.h>
 #include <libgnomeprint/gnome-pgl-private.h>
-#include <libgnomeprint/gp-ps-unicode.h>
 
+/* fixme: */
+#ifdef USE_GNOME_FONT
+#include <libgnomefont/gf-ps-unicode.h>
+#define gp_unicode_from_ps gf_unicode_from_ps
+#else
+#include <libgnomeprint/gp-ps-unicode.h>
+#endif
+
+#ifdef ENABLE_LIBGPA
+#include <libgnomeprint/gnome-print-file.h>
+#endif
 
 static void gnome_print_context_class_init (GnomePrintContextClass *klass);
 
@@ -105,19 +122,54 @@ gnome_print_context_init (GnomePrintContext *pc)
 {
 	pc->gc = gp_gc_new ();
 
+	pc->output = GNOME_PRINT_OUTPUT_NULL;
+	pc->command = NULL;
+	pc->filename = NULL;
 	pc->f = NULL;
 }
 
+/**
+ * gnome_print_context_new_with_paper_size:
+ * @printer: Selected #GnomePrinter
+ * @paper_size: Selected paper size
+ *
+ * This method gives you new #GnomePrintContext object, associated with given
+ * #GnomePrinter. You should use the resulting object as 'black box', without
+ * assuming anything about it's type, as depending on situation appropriate
+ * wrapper context may be used instead of direct driver.
+ *
+ * Returns: The new #GnomePrintContext or %NULL, if there is an error
+ */
+
 GnomePrintContext *
 gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *paper_size)
 {
+	const gchar *driver;
+	
 	g_return_val_if_fail (printer != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_PRINTER (printer), NULL);
 	g_return_val_if_fail (paper_size != NULL, NULL);
 
-	/* This is just TEMPORARY !*/
+#ifdef ENABLE_LIBGPA
+	/* If print to file pop up a dialog */
+	if ((printer->print_to_file) && (gnome_print_file_dialog (printer) != 0)) return NULL;
 
-	if (strcmp (printer->driver, "gnome-print-ps") == 0) {
+	if (printer->filename == NULL)
+		printer->filename = gnome_printer_dup_command (printer);
+
+	/* Get the driver */
+	driver = gnome_printer_const_get (printer, "Driver");
+	if (!driver || *driver == 0) {
+		g_warning ("Could not get the Driver from the Printer.\n");
+		return NULL;
+	}
+
+#else	
+	driver = printer->driver;
+#endif	
+
+        /* This is just TEMPORARY !, we will prolly use dlopen */
+	if (strcmp (driver, "gnome-print-ps") == 0) {
 		GnomePrintPs *ps;
 		GnomePrintContext *frgba;
 		ps = gnome_print_ps_new (printer);
@@ -126,12 +178,10 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 		frgba = gnome_print_frgba_new ((GnomePrintContext *) ps);
 		gtk_object_unref (GTK_OBJECT (ps));
 		return frgba ? frgba : NULL;
-	}
-	if (strcmp (printer->driver, "gnome-print-ps-rgb") == 0) {
+	} else 	if (strcmp (driver, "gnome-print-ps-rgb") == 0) {
 		GnomePrintPs *ps = gnome_print_ps_new (printer);
 		return ps ? GNOME_PRINT_CONTEXT (ps) : NULL;
-	}
-	if (strcmp (printer->driver, "gnome-print-ps2") == 0) {
+	} else if (strcmp (driver, "gnome-print-ps2") == 0) {
 		GnomePrintPs2 *ps2;
 		GnomePrintContext *frgba;
 		ps2 = gnome_print_ps2_new (printer);
@@ -140,17 +190,17 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
 		frgba = gnome_print_frgba_new ((GnomePrintContext *) ps2);
 		gtk_object_unref (GTK_OBJECT (ps2));
 		return frgba ? frgba : NULL;
-	}
-	if (strcmp (printer->driver, "gnome-print-pdf") == 0) {
+	} else  if (strcmp (driver, "gnome-print-pdf") == 0) {
 		GnomePrintPdf *pdf = gnome_print_pdf_new_with_paper (printer, paper_size);
 		return pdf ? GNOME_PRINT_CONTEXT (pdf) : NULL;
-	}
-
-	if (strcmp (printer->driver, "gnome-print-pclr") == 0) {
+	} else 	if (strcmp (driver, "gnome-print-pclr") == 0) {
 		return gnome_print_pclr_new (printer, paper_size, 300);
-	}
-	if (strcmp (printer->driver, "gnome-print-pclv") == 0) {
+	} else 	if (strcmp (driver, "gnome-print-pclv") == 0) {
 		return gnome_print_pclv_new (printer, paper_size, 300);
+	} else 	if (strcmp (driver, "gnome-print-fax") == 0) {
+		return gnome_print_fax_new (printer, paper_size, 100);
+	} else {
+		g_warning ("Could not determine the driver to call ..\n");
 	}
 
 #if 0
@@ -169,12 +219,25 @@ gnome_print_context_new_with_paper_size (GnomePrinter *printer, const char *pape
  * 
  * However, this will do for now, and saves making extra classes.
 */
+
+/**
+ * gnome_print_context_new:
+ * @printer: Selected #GnomePrinter
+ *
+ * This method gives you new #GnomePrintContext object, associated with given
+ * #GnomePrinter. You should use the resulting object as 'black box', without
+ * assuming anything about it's type, as depending on situation appropriate
+ * wrapper context may be used instead of direct driver.
+ *
+ * Returns: The new #GnomePrintContext or %NULL, if there is an error
+ */
+
 GnomePrintContext *
 gnome_print_context_new (GnomePrinter *printer)
 {
 	g_return_val_if_fail (printer != NULL, NULL);
 
-	return gnome_print_context_new_with_paper_size (printer, "US-Letter");
+	return gnome_print_context_new_with_paper_size (printer, gnome_paper_name_default ());
 }
 
 static void
@@ -186,6 +249,24 @@ gnome_print_context_finalize (GtkObject *object)
 
 	gp_gc_unref (pc->gc);
 
+	/* If files are still open, we'll delete them */
+	if (pc->f) {
+		switch (pc->output) {
+		case GNOME_PRINT_OUTPUT_FILE:
+		case GNOME_PRINT_OUTPUT_PROGRAM:
+			fclose (pc->f);
+			unlink (pc->filename);
+			break;
+		case GNOME_PRINT_OUTPUT_PIPE:
+			pclose (pc->f);
+			break;
+		default:
+			break;
+		}
+	}
+	if (pc->filename) g_free (pc->filename);
+	if (pc->command) g_free (pc->command);
+
 	(* GTK_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -476,15 +557,13 @@ gnome_print_show (GnomePrintContext *pc, char const *text)
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (text != NULL, GNOME_PRINT_ERROR_BADVALUE);
 
-	if (PRINT_CLASS (pc)->show_sized)
-		return (* PRINT_CLASS (pc)->show_sized) (pc, text, strlen (text));
-
-	return GNOME_PRINT_OK;
+	return gnome_print_show_sized (pc, text, strlen (text));
 }
 
 int
 gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
 {
+	const GnomeFont *font;
 	const char *invalid;
 
 	g_return_val_if_fail (pc != NULL, -1);
@@ -493,13 +572,21 @@ gnome_print_show_sized (GnomePrintContext *pc, char const *text, int bytes)
 
 	if (bytes < 1) return GNOME_PRINT_OK;
 
-	if (!g_utf8_validate (text, bytes, &invalid))
-		g_warning ("Could not UTF8 Validate *%s*", text);
-	
 	g_return_val_if_fail (g_utf8_validate (text, bytes, &invalid), GNOME_PRINT_ERROR_TEXTCORRUPT);
 
-	if (PRINT_CLASS (pc)->show_sized)
+	/* fixme: */
+	if (!gp_gc_has_currentpoint (pc->gc)) return GNOME_PRINT_ERROR_NOCURRENTPOINT;
+	font = gp_gc_get_font (pc->gc);
+	if (!font) return -1;
+
+	if (PRINT_CLASS (pc)->glyphlist) {
+		GnomeGlyphList *gl;
+		gl = gnome_glyphlist_from_text_sized_dumb ((GnomeFont *) font, gp_gc_get_rgba (pc->gc), 0.0, 0.0, text, bytes);
+		gnome_print_glyphlist (pc, gl);
+		gnome_glyphlist_unref (gl);
+	} else if (PRINT_CLASS (pc)->show_sized) {
 		return (* PRINT_CLASS (pc)->show_sized) (pc, text, bytes);
+	}
 
 	return GNOME_PRINT_OK;
 }
@@ -778,11 +865,35 @@ gnome_print_setopacity (GnomePrintContext *pc, double opacity)
 int
 gnome_print_context_close (GnomePrintContext *pc)
 {
+	gint ret;
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	if (PRINT_CLASS (pc)->close)
-		return PRINT_CLASS(pc)->close (pc);
+	if (PRINT_CLASS (pc)->close) {
+		ret = PRINT_CLASS(pc)->close (pc);
+	}
+
+	/* If files are still open, we'll delete them */
+	if (pc->f) {
+		switch (pc->output) {
+		case GNOME_PRINT_OUTPUT_FILE:
+		case GNOME_PRINT_OUTPUT_PROGRAM:
+			fclose (pc->f);
+			unlink (pc->filename);
+			break;
+		case GNOME_PRINT_OUTPUT_PIPE:
+			pclose (pc->f);
+			break;
+		default:
+			break;
+		}
+		pc->f = NULL;
+	}
+	if (pc->filename) g_free (pc->filename);
+	pc->filename = NULL;
+	if (pc->command) g_free (pc->command);
+	pc->filename = NULL;
 
 	return GNOME_PRINT_OK;
 }
@@ -832,7 +943,6 @@ gnome_print_translate (GnomePrintContext *pc, double x, double y)
 
 /*
  * These functions provide a common interface for writing bytes to the
- *
  * printer.
  */
 
@@ -844,28 +954,59 @@ gnome_print_context_open_file (GnomePrintContext *pc, const char *filename)
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 	g_return_val_if_fail (filename != NULL, -1);
 
-	if (filename[0] == '|')
-	{
+	if (filename[0] == '|') {
+		/* We are pipe */
+		pc->output = GNOME_PRINT_OUTPUT_PIPE;
 		pc->f = popen (filename + 1, "w");
-		pc->is_pipe = TRUE;
-	}
-	else
-	{
-		/* This is not very intelligent, but it worked for me */
-
-		if ((filename[0] == '~') && (filename[1] == '/'))
-		{
-			filename = g_concat_dir_and_file (g_get_home_dir (), &filename[2]);
+	} else if (filename[0] == '*') {
+		/* We are command */
+		/* fixme: */
+		static gint count = 0;
+		guchar fn[64];
+		gint fd = -1;
+		while (fd < 0) {
+			g_snprintf (fn, 64, "/tmp/gp-spool-%d-%d", (gint) time (NULL), count);
+			fd = open (fn, O_WRONLY | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+			g_print ("fd is %d\n", fd);
+			if (fd < 0) {
+				switch (errno) {
+				case EEXIST:
+					count++;
+					break;
+				default:
+					g_warning ("Cannot create temporary spoolfile %s", fn);
+					return -1;
+					break;
+				}
+			}
 		}
-		pc->f = fopen (filename, "w");
-		pc->is_pipe = FALSE;
+		pc->output = GNOME_PRINT_OUTPUT_PROGRAM;
+		pc->command = g_strdup (filename + 1);
+		pc->filename = g_strdup (fn);
+		g_print ("command %s filename %s\n", pc->command, pc->filename);
+		pc->f = fdopen (fd, "w");
+		if (pc->f == NULL) {
+			g_warning ("Cannot reopen temporary spoolfile %s", fn);
+			return -1;
+		}
+	} else {
+		/* We are plain regular file */
+		if ((filename[0] == '~') && (filename[1] == '/')) {
+			pc->filename = g_concat_dir_and_file (g_get_home_dir (), &filename[2]);
+		} else if ((filename[0] != '/') && (filename[0] != '.')) {
+			pc->filename = g_concat_dir_and_file (g_get_home_dir (), filename);
+		} else {
+			pc->filename = g_strdup (filename);
+		}
+		pc->output = GNOME_PRINT_OUTPUT_FILE;
+		pc->f = fopen (pc->filename, "w");
 	}
 	return pc->f != NULL;
 }
 
 /* Return number of bytes written */
 int
-gnome_print_context_write_file (GnomePrintContext *pc, const char *buf, size_t size)
+gnome_print_context_write_file (GnomePrintContext *pc, const void *buf, size_t size)
 {
 	g_return_val_if_fail (pc != NULL, 0);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), 0);
@@ -903,19 +1044,41 @@ gnome_print_context_fprintf (GnomePrintContext *pc, const char *fmt, ...)
 int
 gnome_print_context_close_file (GnomePrintContext *pc)
 {
+	gchar c[256];
+
 	g_return_val_if_fail (pc != NULL, -1);
 	g_return_val_if_fail (GNOME_IS_PRINT_CONTEXT (pc), -1);
 
-	if (pc->f != NULL)
-	{
-		if (pc->is_pipe)
-			return pclose (pc->f);
-		else
-			return fclose (pc->f);
+	if (pc->f != NULL) {
+		switch (pc->output) {
+		case GNOME_PRINT_OUTPUT_FILE:
+			if (fclose (pc->f)) g_warning ("Cannot close output file %s", pc->filename);
+			break;
+		case GNOME_PRINT_OUTPUT_PROGRAM:
+			if (fclose (pc->f)) {
+				g_warning ("Cannot close temporary output file %s", pc->filename);
+				perror ("??");
+				break;
+			}
+			g_snprintf (c, 256, pc->command, pc->filename);
+			g_print ("About to execute %s\n", c);
+			if (system (c)) g_warning ("Cannot execute command %s", c);
+			unlink (pc->filename);
+			break;
+		case GNOME_PRINT_OUTPUT_PIPE:
+			if (pclose (pc->f) == -1) g_warning ("Cannot close pipe");
+			break;
+		default:
+			break;
+		}
 		pc->f = NULL;
 	}
-	else
-		return 0;
+	if (pc->filename) g_free (pc->filename);
+	pc->filename = NULL;
+	if (pc->command) g_free (pc->command);
+	pc->filename = NULL;
+
+	return 0;
 }
 
 void
@@ -1041,6 +1204,7 @@ gnome_print_vpath (GnomePrintContext * gpc, ArtVpath * vpath, gboolean append)
 
 /*
  * WARNING: EXPERIMENTAL
+ * This is also complete crap and we'll remove it soon
  */
 
 int
@@ -1056,32 +1220,35 @@ gnome_print_glyphlist (GnomePrintContext * pc, GnomeGlyphList * glyphlist)
 	} else {
 		GnomePosGlyphList * pgl;
 		gdouble affine[6];
-		GSList * l;
+		gint si;
+
 		art_affine_identity (affine);
 		pgl = gnome_pgl_from_gl (glyphlist, affine, 0);
-		for (l = pgl->strings; l != NULL; l = l->next) {
-			GnomePosString *string;
+		for (si = 0; si < pgl->num_strings; si++) {
+			GnomePosString * ps;
 			GnomeFont *font;
 			GnomeFontFace *face;
+			gdouble r, g, b, a;
 			gint i;
-			string = (GnomePosString *) l->data;
-			font = (GnomeFont *) gnome_rfont_get_font (string->rfont);
+
+			ps = pgl->strings + si;
+			font = (GnomeFont *) gnome_rfont_get_font (ps->rfont);
 			face = (GnomeFontFace *) gnome_font_get_face (font);
 			gnome_print_setfont (pc, font);
-			for (i = 0; i < string->length; i++) {
-				gdouble r, g, b, a;
+			r = ((ps->color >> 24) & 0xff) / 255.0;
+			g = ((ps->color >> 16) & 0xff) / 255.0;
+			b = ((ps->color >>  8) & 0xff) / 255.0;
+			a = ((ps->color >>  0) & 0xff) / 255.0;
+			gnome_print_setrgbcolor (pc, r, g, b);
+			gnome_print_setopacity (pc, a);
+			for (i = ps->start; i < ps->start + ps->length; i++) {
 				const gchar *psname;
 				gchar utf[6];
 				gint len;
+
 				/* fixme: We need currentpoint here :( */
-				gnome_print_moveto (pc, string->glyphs[i].x, string->glyphs[i].y);
-				r = ((string->glyphs[i].color >> 24) & 0xff) / 255.0;
-				g = ((string->glyphs[i].color >> 16) & 0xff) / 255.0;
-				b = ((string->glyphs[i].color >>  8) & 0xff) / 255.0;
-				a = ((string->glyphs[i].color >>  0) & 0xff) / 255.0;
-				gnome_print_setrgbcolor (pc, r, g, b);
-				gnome_print_setopacity (pc, a);
-				psname = gnome_font_face_get_glyph_ps_name (face, string->glyphs[i].glyph);
+				gnome_print_moveto (pc, pgl->glyphs[i].x, pgl->glyphs[i].y);
+				psname = gnome_font_face_get_glyph_ps_name (face, pgl->glyphs[i].glyph);
 				len = g_unichar_to_utf8 (gp_unicode_from_ps (psname), utf);
 				gnome_print_show_sized (pc, utf, len);
 			}
