@@ -2,6 +2,7 @@
 #include <string.h>
 #include <math.h>
 #include <libgnomeprint/gnome-font.h>
+#include <libgnomeprint/gnome-font-private.h>
 #include <libgnomeprint/gnome-text.h>
 
 typedef struct _GnomeTextFontFamily GnomeTextFontFamily;
@@ -292,7 +293,7 @@ gnome_text_layout_new (const guchar *text, GnomeTextAttrEl *attrs)
 	      if (last_glyph >= 0)
 		{
 
-		  for (lig = unsized->ligs[last_glyph];
+		  for (lig = unsized->private->ligs[last_glyph];
 		       lig != NULL;
 		       lig = lig->next)
 		    if (lig->succ == glyph)
@@ -301,7 +302,7 @@ gnome_text_layout_new (const guchar *text, GnomeTextAttrEl *attrs)
 		      }
 
 		  if (lig == NULL)
-		    x += floor (gnome_font_unsized_kern (unsized,
+		    x += floor (gnome_font_face_get_glyph_kerning (unsized,
 							 last_glyph, glyph) *
 				scale_factor + 0.5);
 		}
@@ -318,7 +319,7 @@ gnome_text_layout_new (const guchar *text, GnomeTextAttrEl *attrs)
 		  glyphs[n_glyphs].glyph_num = glyph;
 		  glyphs[n_glyphs].x = x;
 
-		  x += floor (gnome_font_unsized_get_width (unsized, glyph) *
+		  x += floor (gnome_font_face_get_glyph_width (unsized, glyph) *
 			      scale_factor + 0.5);
 
 		  x += track_val;
@@ -339,8 +340,8 @@ gnome_text_layout_new (const guchar *text, GnomeTextAttrEl *attrs)
 		     table with the ligated glyph */
 		  glyph = lig->lig;
 		  glyphs[n_glyphs - 1].glyph_num = glyph;
-		  x += floor ((gnome_font_unsized_get_width (unsized, glyph) - 
-			       gnome_font_unsized_get_width (unsized,
+		  x += floor ((gnome_font_face_get_glyph_width (unsized, glyph) - 
+			       gnome_font_face_get_glyph_width (unsized,
 							     last_glyph)) *
 			      scale_factor + 0.5);
 		  last_glyph = glyph;
@@ -414,7 +415,7 @@ gnome_text_intern_font (GnomeFontUnsized *unsized)
       font_tab = g_new (GnomeFontUnsized *, n_font_tab_max);
     }
 
-  val = g_hash_table_lookup (font_hash, unsized->font_name);
+  val = g_hash_table_lookup (font_hash, gnome_font_face_get_ps_name (unsized));
 
   if (val != NULL)
     return GPOINTER_TO_INT (val);
@@ -426,7 +427,7 @@ gnome_text_intern_font (GnomeFontUnsized *unsized)
 
   font_tab[font_index] = unsized;
 
-  g_hash_table_insert (font_hash, unsized->font_name,
+  g_hash_table_insert (font_hash, (gpointer) gnome_font_face_get_ps_name (unsized),
 		       GINT_TO_POINTER (font_index));
   return font_index;
 }
@@ -566,7 +567,7 @@ gnome_text_unicode_to_glyph (GnomeTextFontListHandle fontlist,
 	}
       if (unsized != NULL)
 	{
-	  glyph = gnome_font_unsized_get_glyph (unsized, unicode);
+	  glyph = gnome_font_face_lookup_default (unsized, unicode);
 	  if (glyph != -1)
 	    {
 	      *p_font = fhandle;
@@ -582,7 +583,7 @@ gnome_text_unicode_to_glyph (GnomeTextFontListHandle fontlist,
 char *
 gnome_text_get_font_name (GnomeTextFontHandle font)
 {
-  return font_tab[font]->font_name;
+  return (char *) gnome_font_face_get_ps_name (font_tab[font]);
 }
 
 GnomeFontUnsized *
@@ -594,7 +595,7 @@ gnome_text_get_font (GnomeTextFontHandle font)
 int
 gnome_text_get_width (GnomeTextFontHandle font, int glyph)
 {
-  return font_tab[font]->widths[glyph];
+  return gnome_font_face_get_glyph_width (font_tab[font], glyph);
 }
 
 /* A simple, high speed justification algorithm. Uses the greedy
